<?php
/**
 * Processor for Picasa. This extends the Photonic_Processor class and defines methods local to Picasa.
 *
 * @package Photonic
 * @subpackage Extensions
 */

class Photonic_Picasa_Processor extends Photonic_OAuth2_Processor {
	function __construct() {
		parent::__construct();
		global $photonic_picasa_client_id, $photonic_picasa_client_secret, $photonic_picasa_disable_title_link, $photonic_picasa_refresh_token;
		$this->client_id = $photonic_picasa_client_id;
		$this->client_secret = $photonic_picasa_client_secret;
		$this->provider = 'picasa';
		$this->oauth_version = '2.0';
		$this->response_type = 'code';
		$this->scope = 'https://picasaweb.google.com/data/';
		$this->link_lightbox_title = empty($photonic_picasa_disable_title_link);

		$cookie = Photonic::parse_cookie();
		global $photonic_picasa_allow_oauth;
		$this->oauth_done = false;
		$this->perform_back_end_authentication($photonic_picasa_refresh_token);

		if ($photonic_picasa_allow_oauth && isset($cookie['picasa']) && isset($cookie['picasa']['oauth_token']) && isset($cookie['picasa']['oauth_refresh_token'])) { // OAuth2, so no Access token secret
			if ($this->is_token_expired($cookie['picasa'])) {
				$this->refresh_token('picasa', $cookie['picasa']['oauth_refresh_token']);
				$cookie = Photonic::parse_cookie(); // Refresh the cookie object based on the results of the refresh token
				if ($this->is_token_expired($cookie['picasa'])) { // Tried refreshing, but didn't work
					$this->oauth_done = false;
				}
				else {
					$this->oauth_done = true;
				}
			}
			else {
				$this->oauth_done = true;
			}
		}
		else if (!isset($cookie['picasa']) || !isset($cookie['picasa']['oauth_token'])) {
			$this->oauth_done = false;
		}
		else if (isset($cookie['picasa']) && !isset($cookie['picasa']['oauth_refresh_token'])) {
			if (!$this->is_token_expired($cookie['picasa'])) {
				$this->oauth_done = true;
			}
			else {
				$this->refresh_token('picasa', $cookie['picasa']['oauth_refresh_token']);
				$cookie = Photonic::parse_cookie(); // Refresh the cookie object based on the results of the refresh token
				if ($this->is_token_expired($cookie['picasa'])) { // Tried refreshing, but didn't work
					$this->oauth_done = false;
				}
				else {
					$this->oauth_done = true;
				}
			}
		}
	}

	/**
	 *
	 * user_id
	 * kind
	 * album
	 * max_results
	 *
	 * thumb_size
	 * columns
	 * shorten caption
	 * show caption
	 *
	 * @param array $attr
	 * @return string
	 */
	function get_gallery_images($attr = array()) {
		$attr = array_merge(array(
			'style' => 'default',
			'show_captions' => false,
			'crop' => true,
			'display' => 'in-page',
			'max_results' => 1000,
			'thumbsize' => 75,
			'filter' => '',
		), $attr);
		extract($attr);

		if (!isset($user_id) || (isset($user_id) && trim($user_id) == '')) {
			return '';
		}

		if (!isset($view)) {
			$view = null;
		}

		$query_url = 'https://picasaweb.google.com/data/feed/api/user/'.$user_id;

		if (isset($album) && trim($album) != '') {
			$query_url .= '/album/'.urlencode($album);
		}

		if (isset($albumid) && trim($albumid) != '') {
			$query_url .= '/albumid/'.urlencode($albumid);
		}

		$to_add = array();
		if (isset($kind) && trim($kind) != '' && in_array(trim($kind), array('album', 'photo', 'tag'))) {
			$kind = trim($kind);
			$to_add[] = 'kind='.$kind;
		}
		else {
			$kind = '';
		}

		if (!isset($view) || $view == null) {
			if ($kind == 'album') {
				$view = 'album';
			}
			else if ($kind == '') {
				if (!isset($album) && !isset($albumid)) {
					$view = 'album';
				}
			}
		}

		global $photonic_archive_thumbs;
		if (is_archive()) {
			if (isset($photonic_archive_thumbs) && !empty($photonic_archive_thumbs)) {
				if (isset($max_results) && $photonic_archive_thumbs < $max_results) {
					$this->show_more_link = true;
					$to_add[] = 'max-results='.$photonic_archive_thumbs;
				}
				else if (isset($max_results)) {
					$to_add[] = 'max-results='.$max_results;
				}
			}
			else if (isset($max_results)) {
				$to_add[] = 'max-results='.$max_results;
			}
		}
		else if (isset($max_results)) {
			$to_add[] = 'max-results='.$max_results;
		}

		if (!empty($authkey)) {
			$to_add[] = 'authkey='.trim($authkey);
		}

		if (isset($thumbsize) && trim($thumbsize) != '') {
			$to_add[] = 'thumbsize='.trim($thumbsize);
		}
		else {
			$to_add[] = 'thumbsize=75';
		}

		$to_add[] = 'imgmax=1600u';

		$query_url .= '?'.implode('&', $to_add);

		global $photonic_picasa_allow_oauth;
		$ret = '';
		if ($photonic_picasa_allow_oauth && !$this->oauth_done && $display != 'popup') {
			$post_id = get_the_ID();
			$ret .= $this->get_login_box($post_id);
		}

		if (!isset($access) || is_null($access)) {
			$access = array('public');
		}
		else {
			$access = explode(',', $access);
		}
		$attr['access'] = $access;

		if (!isset($protection) || empty($protection)) {
			$protection = array('none');
		}
		else {
			$protection = explode(',', $protection);
		}
		$attr['protection'] = $protection;

		return $ret.$this->make_call($query_url, $display, $view, $attr);
	}

	function make_call($query_url, $display, $view, $attr) {
		global $photonic_picasa_allow_oauth, $photonic_picasa_refresh_token;
		extract($attr);

		if (!empty($photonic_picasa_refresh_token) && !empty($this->access_token)) {
			$query_url = add_query_arg('access_token', $this->access_token, $query_url);
			$rss = $this->get_secure_curl_response($query_url);
		}
		else if (isset($photonic_picasa_allow_oauth) && $photonic_picasa_allow_oauth && $this->oauth_done) {
			if (isset($_COOKIE['photonic-' . md5($this->client_secret) . '-oauth-token'])) {
				$query_url = add_query_arg('access_token', $_COOKIE['photonic-' . md5($this->client_secret) . '-oauth-token'], $query_url);
				$response = $this->get_secure_curl_response($query_url);
				if (strlen($response) == 0 || substr($response, 0, 1) != '<') {
					$rss = '';
					if (stripos($response, 'No album found') !== false) {
//						$new_url = $this->get_google_plus_url($query_url);
//						$response = $this->get_secure_curl_response($new_url);
					}
				}
				else {
					$rss = $response;
				}
			}
		}
		else {
			$response = wp_remote_request($query_url);
			if (is_wp_error($response)) {
				$rss = '';
			}
			else if (200 != $response['response']['code']) {
				$rss = '';
			}
			else {
				$rss = $response['body'];
			}
		}

		$this->gallery_index++;

		if ($display != 'popup') {
			$out = "<div class='photonic-picasa-stream photonic-stream' id='photonic-picasa-stream-{$this->gallery_index}'>";
		}
		else {
			if (empty($panel)) {
				$panel = '';
			}
			$out = "<div class='photonic-picasa-panel photonic-panel' id='photonic-picasa-panel-$panel'>";
		}
		if (!isset($columns)) {
			$columns = 'auto';
		}

		if (!isset($panel)) {
			$panel = null;
		}

		$out .= $this->process_response($rss, $view, $display, $columns, $panel, $attr['thumbsize'], $attr['layout'], $attr['filter'], $attr['access'], $attr['protection']);
		$out .= "</div>";
		return $out;
	}

	/**
	 * Reads the output from Picasa and parses it to generate the front-end output.
	 * In a later release this will be streamlined to use DOM-based parsing instead of event-based parsing.
	 *
	 * @param $rss
	 * @param null $view
	 * @param string $display
	 * @param null|string $columns
	 * @param null $panel
	 * @param int $thumb_size
	 * @param string $layout
	 * @param string $filter
	 * @return string
	 */
	function process_response($rss, $view = null, $display = 'in-page', $columns = 'auto', $panel = null, $thumb_size = 75, $layout = 'square', $filter = '', $access, $protection) {
		global $photonic_picasa_photo_title_display, $photonic_picasa_photo_pop_title_display, $photonic_picasa_photos_per_row_constraint,
			$photonic_picasa_photos_constrain_by_count, $photonic_picasa_photos_constrain_by_padding, $photonic_picasa_photos_pop_per_row_constraint,
			$photonic_picasa_photos_pop_constrain_by_count, $photonic_picasa_photos_pop_constrain_by_padding;
		$picasa_result = simplexml_load_string($rss);
		$out = '';
		if ($display == 'popup' && !$this->bypass_popup) {
			$out .= "<div class='photonic-picasa-panel-content photonic-panel-content fix'>";
		}
		if (is_a($picasa_result, 'SimpleXMLElement')) {
			if (isset($picasa_result->entry) && count($picasa_result->entry) > 0) {
				$row_constraints = array('constraint-type' => $photonic_picasa_photos_per_row_constraint, 'padding' => $photonic_picasa_photos_constrain_by_padding, 'count' => $photonic_picasa_photos_constrain_by_count);
				if ($view == 'album' && $display == 'in-page') {
					$objects = $this->build_level_2_objects($picasa_result->entry, $thumb_size, $filter, $access, $protection);
					$out .= $this->generate_level_2_gallery($objects,
						array(
							'row_constraints' => $row_constraints,
							'columns' => $columns,
							'type' => 'albums',
							'singular_type' => 'album',
							'title_position' => $photonic_picasa_photo_title_display,
							'level_1_count_display' => false,
							'layout' => $layout,
							'provider' => 'picasa',
						)
					);
				}
				else {
					if ($display == 'in-page') {
						$title_position = $photonic_picasa_photo_title_display;
					}
					else {
						$row_constraints = array('constraint-type' => $photonic_picasa_photos_pop_per_row_constraint, 'padding' => $photonic_picasa_photos_pop_constrain_by_padding, 'count' => $photonic_picasa_photos_pop_constrain_by_count);
						$title_position = $photonic_picasa_photo_pop_title_display;
					}

					$level_2_meta = $picasa_result->children('openSearch',1);
					$level_2_meta = array(
						'total' => $level_2_meta->totalResults,
						'start' => $level_2_meta->startIndex,
						'end' => $level_2_meta->startIndex + $level_2_meta->itemsPerPage > $level_2_meta->totalResults ? $level_2_meta->totalResults : $level_2_meta->startIndex + $level_2_meta->itemsPerPage,
						'per-page' => $level_2_meta->itemsPerPage,
					);

					$objects = $this->build_level_1_objects($picasa_result->entry);
					$out .= $this->generate_level_1_gallery($objects,
						array(
							'title_position' => $title_position,
							'row_constraints' => $row_constraints,
							'columns' => $columns,
							'panel' => $panel,
							'layout' => $layout,
							'display' => $display,
							'parent' => 'album',
							'level_2_meta' => $level_2_meta,
						)
					);
				}
			}
		}

		if ($out != '') {
			if ($this->show_more_link && !$this->bypass_popup) {
				$out .= $this->more_link_button(get_permalink().'#photonic-picasa-stream-'.$this->gallery_index);
			}
			if ($display == 'popup' && !$this->bypass_popup) {
//				$out .= $this->get_popup_lightbox();
				$out .= "</div>";
			}
		}
		return $out;
	}

	function build_level_1_objects($photos) {
		global $photonic_picasa_use_desc;
		$objects = array();
		foreach ($photos as $entry) {
			$media_photo = $entry->children('media', 1);
			$media_photo = $media_photo->group;
			if (stripos($media_photo->content->attributes()->type, 'video') !== false) {
				continue;
			}
			$gphoto_photo = $entry->children('gphoto', 1);
			$object = array();
			$object['thumbnail'] = $media_photo->thumbnail->attributes()->url;
			$object['main_image'] = $media_photo->content->attributes()->url;
			if (isset($entry->link)) {
				foreach ($entry->link as $link) {
					$attributes = $link->attributes();
					if (isset($attributes['type']) && $attributes['type'] == 'text/html' && isset($attributes['href']) && isset($attributes['rel'])) {
						if ((stripos($attributes['rel'], 'http://schemas.google.com/photos') === 0 || stripos($attributes['rel'], 'http://schemas.google.com/photos') === 0) && stripos($attributes['rel'], '#canonical')) {
							$object['main_page'] = $attributes['href'];
							break;
						}
					}
				}
			}
			if (!isset($object['main_page'])) {
				$object['main_page'] = $object['main_image'];
			}

			if ($photonic_picasa_use_desc == 'desc' || ($photonic_picasa_use_desc == 'desc-title' && !empty($entry->summary))) {
				$object['title'] = esc_attr($entry->summary);
			}
			else {
				$object['title'] = esc_attr($entry->title);
			}
			$object['alt_title'] = $object['title'];
			$object['description'] = esc_attr($entry->summary);
			$object['id'] = "{$gphoto_photo->id}";

			$objects[] = $object;
		}
		return $objects;
	}

	function build_level_2_objects($albums, $thumb_size, $filter, $access, $protection) {
		global $photonic_picasa_use_desc;
		$objects = array();
		$filters = empty($filter) ? array() : explode(',', $filter);

		foreach ($albums as $entry) {
			$media_photo = $entry->children('media', 1);
			$media_photo = $media_photo->group;
			$gphoto_photo = $entry->children('gphoto', 1);

			if (!empty($filters) && !in_array($gphoto_photo->id, $filters) && !in_array($gphoto_photo->name, $filters)) {
				continue;
			}
			if (!in_array($gphoto_photo->access, $access)) {
				continue;
			}

			$object = array();
			$object['id_1'] = "{$gphoto_photo->id}";
			$object['id_2'] = "{$gphoto_photo->user}";

			$object['thumbnail'] = $media_photo->thumbnail->attributes()->url;
			$object['tile_image'] = $media_photo->content->attributes()->url;
			if (isset($entry->link)) {
				foreach ($entry->link as $link) {
					$attributes = $link->attributes();
					if (isset($attributes['type']) && $attributes['type'] == 'text/html' && isset($attributes['href']) && isset($attributes['rel'])) {
						if (((stripos($attributes['rel'], 'http://schemas.google.com/photos') === 0 || stripos($attributes['rel'], 'http://schemas.google.com/photos') === 0) && stripos($attributes['rel'], '#canonical')) || $attributes['rel'] == 'alternate') {
							$object['main_page'] = $attributes['href'];
							break;
						}
					}
				}
			}
			if (!isset($object['main_page'])) {
				$object['main_page'] = $media_photo->content->attributes()->url;
			}
			if ($photonic_picasa_use_desc == 'desc' || ($photonic_picasa_use_desc == 'desc-title' && !empty($entry->summary))) {
				$object['title'] = esc_attr($entry->summary);
			}
			else {
				$object['title'] = esc_attr($entry->title);
			}
			$object['counter'] = $gphoto_photo->numphotos;
			$object['data_attributes'] = array('thumb-size' => $thumb_size);

			if (in_array('authkey', $protection) && $gphoto_photo->access == 'private' && stripos($object['main_page'], '?authkey=') !== FALSE) {
				$object['passworded'] = 1;
				$object['classes'] = array('photonic-picasa-passworded', 'photonic-picasa-passworded-authkey');
			}

			$objects[] = $object;
		}
		return $objects;
	}

	/**
	 * Access Token URL
	 *
	 * @return string
	 */
	public function access_token_URL() {
		return 'https://accounts.google.com/o/oauth2/token';
	}

	public function authentication_url() {
		return 'https://accounts.google.com/o/oauth2/auth';
	}

	function parse_token($response) {
		$token = array();
		if (!is_wp_error($response) && is_array($response)) {
			$body = $response['body'];
			$body = json_decode($body);
			$token['oauth_token'] = $body->access_token;
			$token['oauth_token_type'] = $body->token_type;
			$token['oauth_token_created'] = time();
			$token['oauth_token_expires'] = $body->expires_in;
		}
		return $token;
	}

	function get_secure_curl_response($query_url) {
		$cert = trailingslashit(PHOTONIC_PATH).'include/misc/cacert.crt';
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $query_url);
		curl_setopt($ch, CURLOPT_HEADER, 0); // Don’t return the header, just the html
		curl_setopt($ch, CURLOPT_CAINFO, $cert); // Set the location of the CA-bundle
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // Return contents as a string

		$response = curl_exec ($ch);
		curl_close($ch);
		return $response;
	}

	/**
	 * @param $photonic_picasa_refresh_token
	 */
	public function perform_back_end_authentication($photonic_picasa_refresh_token) {
		$photonic_authentication = get_option('photonic_authentication');
		if (!isset($photonic_authentication)) {
			$photonic_authentication = array();
		}
		if (!isset($photonic_authentication['picasa']) && !empty($photonic_picasa_refresh_token)) {
			$token = $this->get_access_token_from_refresh('picasa', $photonic_picasa_refresh_token, true);
		}
		else if (isset($photonic_authentication['picasa'])) {
			$token = $photonic_authentication['picasa'];
			if (isset($token)) {
				if ($this->is_token_expired($token)) {
					$token = $this->get_access_token_from_refresh('picasa', $photonic_picasa_refresh_token, true);
				}
			}
		}
		if (!empty($token)) {
			$this->access_token = $token['oauth_token'];
		}
	}
}
